@@ -30,9 +30,16 @@ from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, Tenso
 from torch.utils.data.distributed import DistributedSampler
 from tqdm.auto import tqdm, trange
 from tqdm.contrib import tenumerate
+from transformers.optimization import (
+    get_constant_schedule,
+    get_constant_schedule_with_warmup,
+    get_linear_schedule_with_warmup,
+    get_cosine_schedule_with_warmup,
+    get_cosine_with_hard_restarts_schedule_with_warmup,
+    get_polynomial_decay_schedule_with_warmup,
+)
+from transformers.optimization import AdamW, Adafactor
 from transformers import (
-    WEIGHTS_NAME,
-    AdamW,
     AlbertConfig,
     AlbertTokenizer,
     BertConfig,
@@ -46,6 +53,8 @@ from transformers import (
     ElectraTokenizer,
     FlaubertConfig,
     FlaubertTokenizer,
+    LayoutLMConfig,
+    LayoutLMTokenizer,
     LongformerConfig,
     LongformerForSequenceClassification,
     LongformerTokenizer,
@@ -54,13 +63,13 @@ from transformers import (
     MobileBertTokenizer,
     RobertaConfig,
     RobertaTokenizer,
+    WEIGHTS_NAME,
     XLMConfig,
     XLMRobertaConfig,
     XLMRobertaTokenizer,
     XLMTokenizer,
     XLNetConfig,
     XLNetTokenizer,
-    get_linear_schedule_with_warmup,
 )
 from transformers.convert_graph_to_onnx import convert, quantize
 
@@ -74,6 +83,7 @@ from simpletransformers.classification.transformer_models.bert_model import Bert
 from simpletransformers.classification.transformer_models.camembert_model import CamembertForSequenceClassification
 from simpletransformers.classification.transformer_models.distilbert_model import DistilBertForSequenceClassification
 from simpletransformers.classification.transformer_models.flaubert_model import FlaubertForSequenceClassification
+from simpletransformers.classification.transformer_models.layoutlm_model import LayoutLMForSequenceClassification
 from simpletransformers.classification.transformer_models.roberta_model import RobertaForSequenceClassification
 from simpletransformers.classification.transformer_models.xlm_model import XLMForSequenceClassification
 from simpletransformers.classification.transformer_models.xlm_roberta_model import XLMRobertaForSequenceClassification
@@ -130,12 +140,13 @@ class ClassificationModel:
             "distilbert": (DistilBertConfig, DistilBertForSequenceClassification, DistilBertTokenizer),
             "electra": (ElectraConfig, ElectraForSequenceClassification, ElectraTokenizer),
             "flaubert": (FlaubertConfig, FlaubertForSequenceClassification, FlaubertTokenizer),
+            "layoutlm": (LayoutLMConfig, LayoutLMForSequenceClassification, LayoutLMTokenizer),
             "longformer": (LongformerConfig, LongformerForSequenceClassification, LongformerTokenizer),
             "mobilebert": (MobileBertConfig, MobileBertForSequenceClassification, MobileBertTokenizer),
             "roberta": (RobertaConfig, RobertaForSequenceClassification, RobertaTokenizer),
-            "xlnet": (XLNetConfig, XLNetForSequenceClassification, XLNetTokenizer),
             "xlm": (XLMConfig, XLMForSequenceClassification, XLMTokenizer),
             "xlmroberta": (XLMRobertaConfig, XLMRobertaForSequenceClassification, XLMRobertaTokenizer),
+            "xlnet": (XLNetConfig, XLNetForSequenceClassification, XLNetTokenizer),
         }
 
         self.args = self._load_model_args(model_name)
@@ -149,9 +160,12 @@ class ClassificationModel:
             torch.set_num_threads(self.args.thread_count)
 
         if "sweep_config" in kwargs:
+            self.is_sweeping = True
             sweep_config = kwargs.pop("sweep_config")
             sweep_values = sweep_config_to_sweep_values(sweep_config)
             self.args.update_from_dict(sweep_values)
+        else:
+            self.is_sweeping = False
 
         if self.args.manual_seed:
             random.seed(self.args.manual_seed)
@@ -332,22 +346,42 @@ class ClassificationModel:
         if isinstance(train_df, str) and self.args.lazy_loading:
             if self.args.sliding_window:
                 raise ValueError("Lazy loading cannot be used with sliding window.")
+            if self.args.model_type == "layoutlm":
+                raise NotImplementedError("Lazy loading is not implemented for LayoutLM models")
             train_dataset = LazyClassificationDataset(train_df, self.tokenizer, self.args)
         else:
             if self.args.lazy_loading:
                 raise ValueError("Input must be given as a path to a file when using lazy loading")
             if "text" in train_df.columns and "labels" in train_df.columns:
-                train_examples = [
-                    InputExample(i, text, None, label)
-                    for i, (text, label) in enumerate(zip(train_df["text"].astype(str), train_df["labels"]))
-                ]
+                if self.args.model_type == "layoutlm":
+                    train_examples = [
+                        InputExample(i, text, None, label, x0, y0, x1, y1)
+                        for i, (text, label, x0, y0, x1, y1) in enumerate(
+                            zip(
+                                train_df["text"].astype(str),
+                                train_df["labels"],
+                                train_df["x0"],
+                                train_df["y0"],
+                                train_df["x1"],
+                                train_df["y1"],
+                            )
+                        )
+                    ]
+                else:
+                    train_examples = [
+                        InputExample(i, text, None, label)
+                        for i, (text, label) in enumerate(zip(train_df["text"].astype(str), train_df["labels"]))
+                    ]
             elif "text_a" in train_df.columns and "text_b" in train_df.columns:
-                train_examples = [
-                    InputExample(i, text_a, text_b, label)
-                    for i, (text_a, text_b, label) in enumerate(
-                        zip(train_df["text_a"].astype(str), train_df["text_b"].astype(str), train_df["labels"])
-                    )
-                ]
+                if self.args.model_type == "layoutlm":
+                    raise ValueError("LayoutLM cannot be used with sentence-pair tasks")
+                else:
+                    train_examples = [
+                        InputExample(i, text_a, text_b, label)
+                        for i, (text_a, text_b, label) in enumerate(
+                            zip(train_df["text_a"].astype(str), train_df["text_b"].astype(str), train_df["labels"])
+                        )
+                    ]
             else:
                 warnings.warn(
                     "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
@@ -468,10 +502,67 @@ class ClassificationModel:
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
 
-        optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
-        scheduler = get_linear_schedule_with_warmup(
-            optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
-        )
+        if args.optimizer == "AdamW":
+            optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
+        elif args.optimizer == "Adafactor":
+            optimizer = Adafactor(
+                optimizer_grouped_parameters,
+                lr=args.learning_rate,
+                eps=args.adafactor_eps,
+                clip_threshold=args.adafactor_clip_threshold,
+                decay_rate=args.adafactor_decay_rate,
+                beta1=args.adafactor_beta1,
+                weight_decay=args.weight_decay,
+                scale_parameter=args.adafactor_scale_parameter,
+                relative_step=args.adafactor_relative_step,
+                warmup_init=args.adafactor_warmup_init,
+            )
+            print("Using Adafactor for T5")
+        else:
+            raise ValueError(
+                "{} is not a valid optimizer class. Please use one of ('AdamW', 'Adafactor') instead.".format(
+                    args.optimizer
+                )
+            )
+
+        if args.scheduler == "constant_schedule":
+            scheduler = get_constant_schedule(optimizer)
+
+        elif args.scheduler == "constant_schedule_with_warmup":
+            scheduler = get_constant_schedule_with_warmup(optimizer, num_warmup_steps=args.warmup_steps)
+
+        elif args.scheduler == "linear_schedule_with_warmup":
+            scheduler = get_linear_schedule_with_warmup(
+                optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
+            )
+
+        elif args.scheduler == "cosine_schedule_with_warmup":
+            scheduler = get_cosine_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "cosine_with_hard_restarts_schedule_with_warmup":
+            scheduler = get_cosine_with_hard_restarts_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "polynomial_decay_schedule_with_warmup":
+            scheduler = get_polynomial_decay_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                lr_end=args.polynomial_decay_schedule_lr_end,
+                power=args.polynomial_decay_schedule_lr_end,
+            )
+
+        else:
+            raise ValueError("{} is not a valid scheduler.".format(args.scheduler))
 
         if args.n_gpu > 1:
             model = torch.nn.DataParallel(model)
@@ -513,7 +604,9 @@ class ClassificationModel:
             training_progress_scores = self._create_training_progress_scores(multi_label, **kwargs)
 
         if args.wandb_project:
-            wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
+            if not wandb.setup().settings.sweep_id:
+                logger.info(" Initializing WandB run for training.")
+                wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             wandb.watch(self.model)
 
         if self.args.fp16:
@@ -571,7 +664,8 @@ class ClassificationModel:
                 if (step + 1) % args.gradient_accumulation_steps == 0:
                     if self.args.fp16:
                         scaler.unscale_(optimizer)
-                    torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
+                    if args.optimizer == "AdamW":
+                        torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
 
                     if self.args.fp16:
                         scaler.step(optimizer)
@@ -584,14 +678,14 @@ class ClassificationModel:
 
                     if args.logging_steps > 0 and global_step % args.logging_steps == 0:
                         # Log metrics
-                        tb_writer.add_scalar("lr", scheduler.get_lr()[0], global_step)
+                        tb_writer.add_scalar("lr", scheduler.get_last_lr()[0], global_step)
                         tb_writer.add_scalar("loss", (tr_loss - logging_loss) / args.logging_steps, global_step)
                         logging_loss = tr_loss
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(
                                 {
                                     "Training loss": current_loss,
-                                    "lr": scheduler.get_lr()[0],
+                                    "lr": scheduler.get_last_lr()[0],
                                     "global_step": global_step,
                                 }
                             )
@@ -631,7 +725,7 @@ class ClassificationModel:
                             os.path.join(args.output_dir, "training_progress_scores.csv"), index=False,
                         )
 
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(self._get_last_metrics(training_progress_scores))
 
                         if not best_eval_metric:
@@ -717,7 +811,7 @@ class ClassificationModel:
                 report = pd.DataFrame(training_progress_scores)
                 report.to_csv(os.path.join(args.output_dir, "training_progress_scores.csv"), index=False)
 
-                if args.wandb_project:
+                if args.wandb_project or self.is_sweeping:
                     wandb.log(self._get_last_metrics(training_progress_scores))
 
                 if not best_eval_metric:
@@ -829,23 +923,44 @@ class ClassificationModel:
 
         results = {}
         if isinstance(eval_df, str) and self.args.lazy_loading:
+            if self.args.model_type == "layoutlm":
+                raise NotImplementedError("Lazy loading is not implemented for LayoutLM models")
             eval_dataset = LazyClassificationDataset(eval_df, self.tokenizer, self.args)
             eval_examples = None
         else:
             if self.args.lazy_loading:
                 raise ValueError("Input must be given as a path to a file when using lazy loading")
+
             if "text" in eval_df.columns and "labels" in eval_df.columns:
-                eval_examples = [
-                    InputExample(i, text, None, label)
-                    for i, (text, label) in enumerate(zip(eval_df["text"].astype(str), eval_df["labels"]))
-                ]
+                if self.args.model_type == "layoutlm":
+                    eval_examples = [
+                        InputExample(i, text, None, label, x0, y0, x1, y1)
+                        for i, (text, label, x0, y0, x1, y1) in enumerate(
+                            zip(
+                                eval_df["text"].astype(str),
+                                eval_df["labels"],
+                                eval_df["x0"],
+                                eval_df["y0"],
+                                eval_df["x1"],
+                                eval_df["y1"],
+                            )
+                        )
+                    ]
+                else:
+                    eval_examples = [
+                        InputExample(i, text, None, label)
+                        for i, (text, label) in enumerate(zip(eval_df["text"].astype(str), eval_df["labels"]))
+                    ]
             elif "text_a" in eval_df.columns and "text_b" in eval_df.columns:
-                eval_examples = [
-                    InputExample(i, text_a, text_b, label)
-                    for i, (text_a, text_b, label) in enumerate(
-                        zip(eval_df["text_a"].astype(str), eval_df["text_b"].astype(str), eval_df["labels"])
-                    )
-                ]
+                if self.args.model_type == "layoutlm":
+                    raise ValueError("LayoutLM cannot be used with sentence-pair tasks")
+                else:
+                    eval_examples = [
+                        InputExample(i, text_a, text_b, label)
+                        for i, (text_a, text_b, label) in enumerate(
+                            zip(eval_df["text_a"].astype(str), eval_df["text_b"].astype(str), eval_df["labels"])
+                        )
+                    ]
             else:
                 warnings.warn(
                     "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
@@ -962,7 +1077,9 @@ class ClassificationModel:
                 writer.write("{} = {}\n".format(key, str(result[key])))
 
         if self.args.wandb_project and wandb_log and not multi_label and not self.args.regression:
-            wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
+            if not wandb.setup().settings.sweep_id:
+                logger.info(" Initializing WandB run for evaluation.")
+                wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             if not args.labels_map:
                 self.args.labels_map = {i: i for i in range(self.num_labels)}
 
@@ -970,17 +1087,19 @@ class ClassificationModel:
             inverse_labels_map = {value: key for key, value in self.args.labels_map.items()}
 
             truth = [inverse_labels_map[out] for out in out_label_ids]
-            # ROC
-            wandb.log({"roc": wandb.plots.ROC(truth, model_outputs, labels_list)})
-
-            # Precision Recall
-            wandb.log({"pr": wandb.plots.precision_recall(truth, model_outputs, labels_list)})
 
             # Confusion Matrix
             wandb.sklearn.plot_confusion_matrix(
-                truth, [inverse_labels_map[np.argmax(out)] for out in model_outputs], labels=labels_list,
+                truth, [inverse_labels_map[pred] for pred in preds], labels=labels_list,
             )
 
+            if not self.args.sliding_window:
+                # ROC`
+                wandb.log({"roc": wandb.plots.ROC(truth, model_outputs, labels_list)})
+
+                # Precision Recall
+                wandb.log({"pr": wandb.plots.precision_recall(truth, model_outputs, labels_list)})
+
         return results, model_outputs, wrong
 
     def load_and_cache_examples(
@@ -1083,12 +1202,18 @@ class ClassificationModel:
         all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
         all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
 
+        if self.args.model_type == "layoutlm":
+            all_bboxes = torch.tensor([f.bboxes for f in features], dtype=torch.long)
+
         if output_mode == "classification":
             all_label_ids = torch.tensor([f.label_id for f in features], dtype=torch.long)
         elif output_mode == "regression":
             all_label_ids = torch.tensor([f.label_id for f in features], dtype=torch.float)
 
-        dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
+        if self.args.model_type == "layoutlm":
+            dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids, all_bboxes)
+        else:
+            dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
 
         if args.sliding_window and evaluate:
             return dataset, window_counts
@@ -1117,7 +1242,18 @@ class ClassificationModel:
         for metric, func in kwargs.items():
             extra_metrics[metric] = func(labels, preds)
 
-        mismatched = labels != preds
+        if multi_label:
+            threshold_values = self.args.threshold if self.args.threshold else 0.5
+            if isinstance(threshold_values, list):
+                mismatched = labels != [
+                    [self._threshold(pred, threshold_values[i]) for i, pred in enumerate(example)] for example in preds
+                ]
+            else:
+                mismatched = labels != [
+                    [self._threshold(pred, threshold_values) for pred in example] for example in preds
+                ]
+        else:
+            mismatched = labels != preds
 
         if eval_examples:
             wrong = [i for (i, v) in zip(eval_examples, mismatched) if v.any()]
@@ -1215,6 +1351,11 @@ class ClassificationModel:
                     eval_examples = [InputExample(i, text, None, dummy_label) for i, text in enumerate(to_predict)]
             if args.sliding_window:
                 eval_dataset, window_counts = self.load_and_cache_examples(eval_examples, evaluate=True, no_cache=True)
+                preds = np.empty((len(eval_dataset), self.num_labels))
+                if multi_label:
+                    out_label_ids = np.empty((len(eval_dataset), self.num_labels))
+                else:
+                    out_label_ids = np.empty((len(eval_dataset)))
             else:
                 eval_dataset = self.load_and_cache_examples(
                     eval_examples, evaluate=True, multi_label=multi_label, no_cache=True
@@ -1412,7 +1553,12 @@ class ClassificationModel:
 
             # XLM, DistilBERT and RoBERTa don't use segment_ids
             if self.args.model_type != "distilbert":
-                inputs["token_type_ids"] = batch[2] if self.args.model_type in ["bert", "xlnet", "albert"] else None
+                inputs["token_type_ids"] = (
+                    batch[2] if self.args.model_type in ["bert", "xlnet", "albert", "layoutlm"] else None
+                )
+
+        if self.args.model_type == "layoutlm":
+            inputs["bbox"] = batch[4]
 
         return inputs
 
