@@ -1,5 +1,5 @@
-from transformers.configuration_xlm_roberta import XLMRobertaConfig
-from transformers.modeling_xlm_roberta import XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
+from transformers.models.xlm_roberta.configuration_xlm_roberta import XLMRobertaConfig
+from transformers.models.xlm_roberta.modeling_xlm_roberta import XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST
 
 from simpletransformers.classification.transformer_models.roberta_model import RobertaForSequenceClassification
 
