@@ -32,16 +32,27 @@ from tensorboardX import SummaryWriter
 from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
 from torch.utils.data.distributed import DistributedSampler
 from tqdm.auto import tqdm, trange
+from transformers.optimization import (
+    get_constant_schedule,
+    get_constant_schedule_with_warmup,
+    get_linear_schedule_with_warmup,
+    get_cosine_schedule_with_warmup,
+    get_cosine_with_hard_restarts_schedule_with_warmup,
+    get_polynomial_decay_schedule_with_warmup,
+)
+from transformers.optimization import AdamW, Adafactor
 from transformers import (
     WEIGHTS_NAME,
-    AdamW,
     GPT2Config,
     GPT2DoubleHeadsModel,
     GPT2Tokenizer,
     OpenAIGPTConfig,
     OpenAIGPTDoubleHeadsModel,
     OpenAIGPTTokenizer,
-    get_linear_schedule_with_warmup,
+    BlenderbotSmallTokenizer,
+    BlenderbotForConditionalGeneration,
+    BlenderbotTokenizer,
+    BlenderbotConfig,
 )
 
 from simpletransformers.classification.classification_utils import InputExample, convert_examples_to_features
@@ -90,6 +101,8 @@ class ConvAIModel:
         MODEL_CLASSES = {
             "gpt": (OpenAIGPTConfig, OpenAIGPTDoubleHeadsModel, OpenAIGPTTokenizer),
             "gpt2": (GPT2Config, GPT2DoubleHeadsModel, GPT2Tokenizer),
+            "blender-small": (BlenderbotConfig, BlenderbotForConditionalGeneration, BlenderbotSmallTokenizer),
+            "blender": (BlenderbotConfig, BlenderbotForConditionalGeneration, BlenderbotTokenizer),
         }
 
         self.args = self._load_model_args(model_name)
@@ -100,9 +113,12 @@ class ConvAIModel:
             self.args = args
 
         if "sweep_config" in kwargs:
+            self.is_sweeping = True
             sweep_config = kwargs.pop("sweep_config")
             sweep_values = sweep_config_to_sweep_values(sweep_config)
             self.args.update_from_dict(sweep_values)
+        else:
+            self.is_sweeping = False
 
         if self.args.manual_seed:
             random.seed(self.args.manual_seed)
@@ -184,6 +200,9 @@ class ConvAIModel:
             training_details: Average training loss if evaluate_during_training is False or full training progress scores if evaluate_during_training is True
         """  # noqa: ignore flake8"
 
+        if self.args.model_type in ["blender-small", "blender"]:
+            raise ValueError("Fine-tuning of Blender models is not currently supported.")
+
         if args:
             self.args.update_from_dict(args)
 
@@ -302,10 +321,67 @@ class ConvAIModel:
         warmup_steps = math.ceil(t_total * args.warmup_ratio)
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
 
-        optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
-        scheduler = get_linear_schedule_with_warmup(
-            optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
-        )
+        if args.optimizer == "AdamW":
+            optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
+        elif args.optimizer == "Adafactor":
+            optimizer = Adafactor(
+                optimizer_grouped_parameters,
+                lr=args.learning_rate,
+                eps=args.adafactor_eps,
+                clip_threshold=args.adafactor_clip_threshold,
+                decay_rate=args.adafactor_decay_rate,
+                beta1=args.adafactor_beta1,
+                weight_decay=args.weight_decay,
+                scale_parameter=args.adafactor_scale_parameter,
+                relative_step=args.adafactor_relative_step,
+                warmup_init=args.adafactor_warmup_init,
+            )
+            print("Using Adafactor for T5")
+        else:
+            raise ValueError(
+                "{} is not a valid optimizer class. Please use one of ('AdamW', 'Adafactor') instead.".format(
+                    args.optimizer
+                )
+            )
+
+        if args.scheduler == "constant_schedule":
+            scheduler = get_constant_schedule(optimizer)
+
+        elif args.scheduler == "constant_schedule_with_warmup":
+            scheduler = get_constant_schedule_with_warmup(optimizer, num_warmup_steps=args.warmup_steps)
+
+        elif args.scheduler == "linear_schedule_with_warmup":
+            scheduler = get_linear_schedule_with_warmup(
+                optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
+            )
+
+        elif args.scheduler == "cosine_schedule_with_warmup":
+            scheduler = get_cosine_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "cosine_with_hard_restarts_schedule_with_warmup":
+            scheduler = get_cosine_with_hard_restarts_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "polynomial_decay_schedule_with_warmup":
+            scheduler = get_polynomial_decay_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                lr_end=args.polynomial_decay_schedule_lr_end,
+                power=args.polynomial_decay_schedule_lr_end,
+            )
+
+        else:
+            raise ValueError("{} is not a valid scheduler.".format(args.scheduler))
 
         if args.n_gpu > 1:
             model = torch.nn.DataParallel(model)
@@ -386,7 +462,8 @@ class ConvAIModel:
                 if (step + 1) % args.gradient_accumulation_steps == 0:
                     if args.fp16:
                         scaler.unscale_(optimizer)
-                    torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
+                    if args.optimizer == "AdamW":
+                        torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
 
                     if args.fp16:
                         scaler.step(optimizer)
@@ -399,14 +476,14 @@ class ConvAIModel:
 
                     if args.logging_steps > 0 and global_step % args.logging_steps == 0:
                         # Log metrics
-                        tb_writer.add_scalar("lr", scheduler.get_lr()[0], global_step)
+                        tb_writer.add_scalar("lr", scheduler.get_last_lr()[0], global_step)
                         tb_writer.add_scalar("loss", (tr_loss - logging_loss) / args.logging_steps, global_step)
                         logging_loss = tr_loss
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(
                                 {
                                     "Training loss": current_loss,
-                                    "lr": scheduler.get_lr()[0],
+                                    "lr": scheduler.get_last_lr()[0],
                                     "global_step": global_step,
                                 }
                             )
@@ -445,7 +522,7 @@ class ConvAIModel:
                             os.path.join(args.output_dir, "training_progress_scores.csv"), index=False,
                         )
 
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(self._get_last_metrics(training_progress_scores))
 
                         if not best_eval_metric:
@@ -523,7 +600,7 @@ class ConvAIModel:
                 report = pd.DataFrame(training_progress_scores)
                 report.to_csv(os.path.join(args.output_dir, "training_progress_scores.csv"), index=False)
 
-                if args.wandb_project:
+                if args.wandb_project or self.is_sweeping:
                     wandb.log(self._get_last_metrics(training_progress_scores))
 
                 if not best_eval_metric:
@@ -774,20 +851,24 @@ class ConvAIModel:
 
         self._move_model_to_device()
 
-        if not personality:
-            dataset = get_dataset(
-                tokenizer,
-                None,
-                args.cache_dir,
-                process_count=process_count,
-                proxies=self.__dict__.get("proxies", None),
-                interact=True,
-                args=args,
-            )
-            personalities = [dialog["personality"] for dataset in dataset.values() for dialog in dataset]
-            personality = random.choice(personalities)
+        if self.args.model_type in ["blender", "blender-small"]:
+            if not personality:
+                personality = []
         else:
-            personality = [tokenizer.encode(s.lower()) for s in personality]
+            if not personality:
+                dataset = get_dataset(
+                    tokenizer,
+                    None,
+                    args.cache_dir,
+                    process_count=process_count,
+                    proxies=self.__dict__.get("proxies", None),
+                    interact=True,
+                    args=args,
+                )
+                personalities = [dialog["personality"] for dataset in dataset.values() for dialog in dataset]
+                personality = random.choice(personalities)
+            else:
+                personality = [tokenizer.encode(s.lower()) for s in personality]
 
         history = []
         while True:
@@ -795,7 +876,9 @@ class ConvAIModel:
             while not raw_text:
                 print("Prompt should not be empty!")
                 raw_text = input(">>> ")
-            history.append(tokenizer.encode(raw_text))
+            history.append(
+                tokenizer.encode(raw_text) if self.args.model_type not in ["blender", "blender-small"] else raw_text
+            )
             with torch.no_grad():
                 if args.fp16:
                     with amp.autocast():
@@ -804,8 +887,12 @@ class ConvAIModel:
                     out_ids = self.sample_sequence(personality, history, tokenizer, model, args)
             history.append(out_ids)
             history = history[-(2 * args.max_history + 1) :]
-            out_text = tokenizer.decode(out_ids, skip_special_tokens=self.args.skip_special_tokens)
+            if self.args.model_type in ["blender", "blender-small"]:
+                out_text = out_ids
+            else:
+                out_text = tokenizer.decode(out_ids, skip_special_tokens=self.args.skip_special_tokens)
             print(out_text)
+            print(history)
 
     def interact_single(self, message, history, personality=None, encode_history=True):
         """
@@ -984,34 +1071,51 @@ class ConvAIModel:
         return logits
 
     def sample_sequence(self, personality, history, tokenizer, model, args, current_output=None):
-        special_tokens_ids = tokenizer.convert_tokens_to_ids(SPECIAL_TOKENS)
-        if current_output is None:
-            current_output = []
+        if self.args.model_type in ["blender", "blender-small"]:
+            print("Input >>>>>>> ", "\n".join(personality) + "\n" + "\n".join(history))
+            print("---------------------------------")
+            inputs = self.tokenizer(["\n".join(history).strip("\n")], return_tensors="pt")
+            # inputs = self.tokenizer(["\n".join(personality) + "\n" + "\n".join(history)], return_tensors='pt')
+            inputs["input_ids"] = inputs["input_ids"].to(self.device)
+            inputs["attention_mask"] = inputs["attention_mask"].to(self.device)
+            reply_ids = self.model.generate(**inputs)
+            reply = [
+                tokenizer.decode(g, skip_special_tokens=True, clean_up_tokenization_spaces=True) for g in reply_ids
+            ]
+            return (
+                reply[0][10:-8] if self.args.model_type == "blender-small" else reply[0]
+            )  # To remove the "__start__ " and " __end__"
+        else:
+            special_tokens_ids = tokenizer.convert_tokens_to_ids(SPECIAL_TOKENS)
+            if current_output is None:
+                current_output = []
 
-        for i in range(args.max_length):
-            instance = self.build_input_from_segments(personality, history, current_output, tokenizer, with_eos=False)
+            for i in range(args.max_length):
+                instance = self.build_input_from_segments(
+                    personality, history, current_output, tokenizer, with_eos=False
+                )
 
-            input_ids = torch.tensor(instance["input_ids"], device=self.device).unsqueeze(0)
-            token_type_ids = torch.tensor(instance["token_type_ids"], device=self.device).unsqueeze(0)
+                input_ids = torch.tensor(instance["input_ids"], device=self.device).unsqueeze(0)
+                token_type_ids = torch.tensor(instance["token_type_ids"], device=self.device).unsqueeze(0)
 
-            logits = model(input_ids, token_type_ids=token_type_ids)
-            if isinstance(logits, tuple):  # for gpt2 and maybe others
-                logits = logits[0]
-            logits = logits[0, -1, :] / args.temperature
-            logits = self.top_filtering(logits, top_k=args.top_k, top_p=args.top_p)
-            probs = F.softmax(logits, dim=-1)
+                logits = model(input_ids, token_type_ids=token_type_ids)
+                if args.model_type in ["gpt2", "gpt"]:  # for gpt2 and maybe others
+                    logits = logits[0]
+                logits = logits[0, -1, :] / args.temperature
+                logits = self.top_filtering(logits, top_k=args.top_k, top_p=args.top_p)
+                probs = F.softmax(logits, dim=-1)
 
-            prev = torch.topk(probs, 1)[1] if not args.do_sample else torch.multinomial(probs, 1)
-            if i < args.min_length and prev.item() in special_tokens_ids:
-                while prev.item() in special_tokens_ids:
-                    if probs.max().item() == 1:
-                        warnings.warn("Warning: model generating special token with probability 1.")
-                        break  # avoid infinitely looping over special token
-                    prev = torch.multinomial(probs, num_samples=1)
+                prev = torch.topk(probs, 1)[1] if not args.do_sample else torch.multinomial(probs, 1)
+                if i < args.min_length and prev.item() in special_tokens_ids:
+                    while prev.item() in special_tokens_ids:
+                        if probs.max().item() == 1:
+                            warnings.warn("Warning: model generating special token with probability 1.")
+                            break  # avoid infinitely looping over special token
+                        prev = torch.multinomial(probs, num_samples=1)
 
-            if prev.item() in special_tokens_ids:
-                break
-            current_output.append(prev.item())
+                if prev.item() in special_tokens_ids:
+                    break
+                current_output.append(prev.item())
 
         return current_output
 
