@@ -1,8 +1,12 @@
 import torch
 import torch.nn as nn
 from torch.nn import CrossEntropyLoss, MSELoss
-from transformers.modeling_camembert import CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST, CamembertConfig, CamembertModel
-from transformers.modeling_roberta import RobertaClassificationHead, RobertaForSequenceClassification
+from transformers.models.camembert.modeling_camembert import (
+    CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST,
+    CamembertConfig,
+    CamembertModel,
+)
+from transformers.models.roberta.modeling_roberta import RobertaClassificationHead, RobertaForSequenceClassification
 
 
 class CamembertForSequenceClassification(RobertaForSequenceClassification):
