@@ -16,14 +16,25 @@ from torch.nn.utils.rnn import pad_sequence
 from torch.utils.data import DataLoader, Dataset, RandomSampler, SequentialSampler
 from torch.utils.data.distributed import DistributedSampler
 from tqdm.auto import tqdm, trange
+from transformers.optimization import (
+    get_constant_schedule,
+    get_constant_schedule_with_warmup,
+    get_linear_schedule_with_warmup,
+    get_cosine_schedule_with_warmup,
+    get_cosine_with_hard_restarts_schedule_with_warmup,
+    get_polynomial_decay_schedule_with_warmup,
+)
+from transformers.optimization import AdamW, Adafactor
 from transformers import (
-    AdamW,
     AutoConfig,
     AutoModel,
     AutoTokenizer,
     BartConfig,
     BartForConditionalGeneration,
     BartTokenizer,
+    MBartConfig,
+    MBartForConditionalGeneration,
+    MBartTokenizer,
     BertConfig,
     BertForMaskedLM,
     BertModel,
@@ -53,7 +64,6 @@ from transformers import (
     RobertaConfig,
     RobertaModel,
     RobertaTokenizer,
-    get_linear_schedule_with_warmup,
 )
 
 from simpletransformers.config.global_args import global_args
@@ -73,6 +83,7 @@ logger = logging.getLogger(__name__)
 MODEL_CLASSES = {
     "auto": (AutoConfig, AutoModel, AutoTokenizer),
     "bart": (BartConfig, BartForConditionalGeneration, BartTokenizer),
+    "mbart": (MBartConfig, MBartForConditionalGeneration, MBartTokenizer),
     "bert": (BertConfig, BertModel, BertTokenizer),
     "camembert": (CamembertConfig, CamembertModel, CamembertTokenizer),
     "distilbert": (DistilBertConfig, DistilBertModel, DistilBertTokenizer),
@@ -143,9 +154,12 @@ class Seq2SeqModel:
             self.args = args
 
         if "sweep_config" in kwargs:
+            self.is_sweeping = True
             sweep_config = kwargs.pop("sweep_config")
             sweep_values = sweep_config_to_sweep_values(sweep_config)
             self.args.update_from_dict(sweep_values)
+        else:
+            self.is_sweeping = False
 
         if self.args.manual_seed:
             random.seed(self.args.manual_seed)
@@ -179,9 +193,9 @@ class Seq2SeqModel:
         else:
             config_class, model_class, tokenizer_class = MODEL_CLASSES[encoder_type]
 
-        if encoder_decoder_type in ["bart", "marian"]:
+        if encoder_decoder_type in ["bart", "mbart", "marian"]:
             self.model = model_class.from_pretrained(encoder_decoder_name)
-            if encoder_decoder_type == "bart":
+            if encoder_decoder_type in ["bart", "mbart"]:
                 self.encoder_tokenizer = tokenizer_class.from_pretrained(encoder_decoder_name)
             elif encoder_decoder_type == "marian":
                 if self.args.base_marian_model_name:
@@ -396,10 +410,67 @@ class Seq2SeqModel:
         args.warmup_steps = warmup_steps if args.warmup_steps == 0 else args.warmup_steps
 
         # TODO: Use custom optimizer like with BertSum?
-        optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
-        scheduler = get_linear_schedule_with_warmup(
-            optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
-        )
+        if args.optimizer == "AdamW":
+            optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
+        elif args.optimizer == "Adafactor":
+            optimizer = Adafactor(
+                optimizer_grouped_parameters,
+                lr=args.learning_rate,
+                eps=args.adafactor_eps,
+                clip_threshold=args.adafactor_clip_threshold,
+                decay_rate=args.adafactor_decay_rate,
+                beta1=args.adafactor_beta1,
+                weight_decay=args.weight_decay,
+                scale_parameter=args.adafactor_scale_parameter,
+                relative_step=args.adafactor_relative_step,
+                warmup_init=args.adafactor_warmup_init,
+            )
+            print("Using Adafactor for T5")
+        else:
+            raise ValueError(
+                "{} is not a valid optimizer class. Please use one of ('AdamW', 'Adafactor') instead.".format(
+                    args.optimizer
+                )
+            )
+
+        if args.scheduler == "constant_schedule":
+            scheduler = get_constant_schedule(optimizer)
+
+        elif args.scheduler == "constant_schedule_with_warmup":
+            scheduler = get_constant_schedule_with_warmup(optimizer, num_warmup_steps=args.warmup_steps)
+
+        elif args.scheduler == "linear_schedule_with_warmup":
+            scheduler = get_linear_schedule_with_warmup(
+                optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
+            )
+
+        elif args.scheduler == "cosine_schedule_with_warmup":
+            scheduler = get_cosine_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "cosine_with_hard_restarts_schedule_with_warmup":
+            scheduler = get_cosine_with_hard_restarts_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                num_cycles=args.cosine_schedule_num_cycles,
+            )
+
+        elif args.scheduler == "polynomial_decay_schedule_with_warmup":
+            scheduler = get_polynomial_decay_schedule_with_warmup(
+                optimizer,
+                num_warmup_steps=args.warmup_steps,
+                num_training_steps=t_total,
+                lr_end=args.polynomial_decay_schedule_lr_end,
+                power=args.polynomial_decay_schedule_lr_end,
+            )
+
+        else:
+            raise ValueError("{} is not a valid scheduler.".format(args.scheduler))
 
         if (
             args.model_name
@@ -510,7 +581,8 @@ class Seq2SeqModel:
                 if (step + 1) % args.gradient_accumulation_steps == 0:
                     if args.fp16:
                         scaler.unscale_(optimizer)
-                    torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
+                    if args.optimizer == "AdamW":
+                        torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
 
                     if args.fp16:
                         scaler.step(optimizer)
@@ -523,14 +595,14 @@ class Seq2SeqModel:
 
                     if args.logging_steps > 0 and global_step % args.logging_steps == 0:
                         # Log metrics
-                        tb_writer.add_scalar("lr", scheduler.get_lr()[0], global_step)
+                        tb_writer.add_scalar("lr", scheduler.get_last_lr()[0], global_step)
                         tb_writer.add_scalar("loss", (tr_loss - logging_loss) / args.logging_steps, global_step)
                         logging_loss = tr_loss
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(
                                 {
                                     "Training loss": current_loss,
-                                    "lr": scheduler.get_lr()[0],
+                                    "lr": scheduler.get_last_lr()[0],
                                     "global_step": global_step,
                                 }
                             )
@@ -569,7 +641,7 @@ class Seq2SeqModel:
                             os.path.join(args.output_dir, "training_progress_scores.csv"), index=False,
                         )
 
-                        if args.wandb_project:
+                        if args.wandb_project or self.is_sweeping:
                             wandb.log(self._get_last_metrics(training_progress_scores))
 
                         if not best_eval_metric:
@@ -660,7 +732,7 @@ class Seq2SeqModel:
                 report = pd.DataFrame(training_progress_scores)
                 report.to_csv(os.path.join(args.output_dir, "training_progress_scores.csv"), index=False)
 
-                if args.wandb_project:
+                if args.wandb_project or self.is_sweeping:
                     wandb.log(self._get_last_metrics(training_progress_scores))
 
                 if not best_eval_metric:
@@ -836,22 +908,37 @@ class Seq2SeqModel:
 
         all_outputs = []
         # Batching
-        for batch in [
-            to_predict[i : i + self.args.eval_batch_size] for i in range(0, len(to_predict), self.args.eval_batch_size)
-        ]:
+        for batch in tqdm(
+            [
+                to_predict[i : i + self.args.eval_batch_size]
+                for i in range(0, len(to_predict), self.args.eval_batch_size)
+            ],
+            desc="Generating outputs",
+            disable=self.args.silent,
+        ):
             if self.args.model_type == "marian":
-                input_ids = self.encoder_tokenizer.prepare_translation_batch(
+                input_ids = self.encoder_tokenizer.prepare_seq2seq_batch(
                     batch,
                     max_length=self.args.max_seq_length,
-                    pad_to_max_length=True,
+                    padding="max_length",
                     return_tensors="pt",
                     truncation=True,
                 )["input_ids"]
+            elif self.args.model_type in ["mbart"]:
+                input_ids = self.encoder_tokenizer.prepare_seq2seq_batch(
+                    src_texts=batch,
+                    max_length=self.args.max_seq_length,
+                    pad_to_max_length=True,
+                    padding="max_length",
+                    return_tensors="pt",
+                    truncation=True,
+                    src_lang=self.args.src_lang,
+                )["input_ids"]
             else:
                 input_ids = self.encoder_tokenizer.batch_encode_plus(
                     batch,
                     max_length=self.args.max_seq_length,
-                    pad_to_max_length=True,
+                    padding="max_length",
                     return_tensors="pt",
                     truncation=True,
                 )["input_ids"]
@@ -870,6 +957,22 @@ class Seq2SeqModel:
                     top_p=self.args.top_p,
                     num_return_sequences=self.args.num_return_sequences,
                 )
+            elif self.args.model_type in ["mbart"]:
+                tgt_lang_token = self.decoder_tokenizer._convert_token_to_id(self.args.tgt_lang)
+
+                outputs = self.model.generate(
+                    input_ids=input_ids,
+                    decoder_start_token_id=tgt_lang_token,
+                    num_beams=self.args.num_beams,
+                    max_length=self.args.max_length,
+                    length_penalty=self.args.length_penalty,
+                    early_stopping=self.args.early_stopping,
+                    repetition_penalty=self.args.repetition_penalty,
+                    do_sample=self.args.do_sample,
+                    top_k=self.args.top_k,
+                    top_p=self.args.top_p,
+                    num_return_sequences=self.args.num_return_sequences,
+                )
             else:
                 outputs = self.model.generate(
                     input_ids=input_ids,
@@ -965,7 +1068,7 @@ class Seq2SeqModel:
             CustomDataset = args.dataset_class
             return CustomDataset(encoder_tokenizer, decoder_tokenizer, args, data, mode)
         else:
-            if args.model_type in ["bart", "marian"]:
+            if args.model_type in ["bart", "mbart", "marian"]:
                 return SimpleSummarizationDataset(encoder_tokenizer, self.args, data, mode)
             else:
                 return Seq2SeqDataset(encoder_tokenizer, decoder_tokenizer, self.args, data, mode,)
@@ -996,11 +1099,11 @@ class Seq2SeqModel:
             model_to_save = model.module if hasattr(model, "module") else model
             self.save_model_args(output_dir)
 
-            if self.args.model_type in ["bart", "marian"]:
+            if self.args.model_type in ["bart", "mbart", "marian"]:
                 os.makedirs(os.path.join(output_dir), exist_ok=True)
                 model_to_save.save_pretrained(output_dir)
                 self.config.save_pretrained(output_dir)
-                if self.args.model_type == "bart":
+                if self.args.model_type in ["bart", "mbart"]:
                     self.encoder_tokenizer.save_pretrained(output_dir)
             else:
                 os.makedirs(os.path.join(output_dir, "encoder"), exist_ok=True)
@@ -1042,24 +1145,31 @@ class Seq2SeqModel:
             pad_token_id = self.encoder_tokenizer.pad_token_id
             source_ids, source_mask, y = batch["source_ids"], batch["source_mask"], batch["target_ids"]
             y_ids = y[:, :-1].contiguous()
-            lm_labels = y[:, 1:].clone()
-            lm_labels[y[:, 1:] == pad_token_id] = -100
+            labels = y[:, 1:].clone()
+            labels[y[:, 1:] == pad_token_id] = -100
 
             inputs = {
                 "input_ids": source_ids.to(device),
                 "attention_mask": source_mask.to(device),
                 "decoder_input_ids": y_ids.to(device),
-                "lm_labels": lm_labels.to(device),
+                "labels": labels.to(device),
+            }
+        elif self.args.model_type in ["mbart"]:
+            inputs = {
+                "input_ids": batch["input_ids"].to(device),
+                "attention_mask": batch["attention_mask"].to(device),
+                "decoder_input_ids": batch["decoder_input_ids"].to(device),
+                "labels": batch["labels"].to(device),
             }
         else:
-            lm_labels = batch[1]
-            lm_labels_masked = lm_labels.clone()
-            lm_labels_masked[lm_labels_masked == self.decoder_tokenizer.pad_token_id] = -100
+            labels = batch[1]
+            labels_masked = labels.clone()
+            labels_masked[labels_masked == self.decoder_tokenizer.pad_token_id] = -100
 
             inputs = {
                 "input_ids": batch[0].to(device),
-                "decoder_input_ids": lm_labels.to(device),
-                "labels": lm_labels_masked.to(device),
+                "decoder_input_ids": labels.to(device),
+                "labels": labels_masked.to(device),
             }
 
         return inputs
